<?php


namespace Fastrack\Settings;


use Exception;
use Fastrack\Property\Property;
use GuzzleHttp\Client;
use League\OAuth2\Client\Provider\Instagram;
use League\OAuth2\Client\Token\AccessToken;
use SilverStripe\Control\Controller;
use SilverStripe\Control\Director;
use SilverStripe\Forms\HTMLReadonlyField;
use SilverStripe\Forms\LiteralField;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\FieldType\DBDatetime;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Security\Security;
use SilverStripe\SiteConfig\SiteConfig;


/**
 * @property string $AppId
 * @property string $AppSecret
 * @property string $UserName
 * @property string $State
 * @property string $AccessToken
 * @property string $RefreshToken
 * @property string $ResourceOwnerId
 * @property int $Expires
 */
class InstagramToken extends DataObject implements PermissionProvider
{

	private static string $table_name = 'instagram_token';

	private static array $db = [
		'AppId' => 'Varchar',
		'AppSecret' => 'Varchar',
		'UserName' => 'Varchar',
		'State' => 'Varchar',
		'AccessToken' => 'Varchar',
		'RefreshToken' => 'Varchar',
		'ResourceOwnerId' => 'Varchar',
		'Expires' => 'Int',
	];

	private static array $has_one = [
		'SiteConfig' => SiteConfig::class,
	];

	private static array $summary_fields = [
		'Username',
	];

	public function getTitle()
	{
		return $this->UserName;
	}


	public function getProvider(): Instagram
	{
		$redirectUrl = Controller::join_links([
			Director::absoluteBaseURL(),
			'instagram',
			$this->UserName,
			'auth',
		]);
		return new Instagram([
			'clientId' => $this->AppId,
			'clientSecret' => $this->AppSecret,
			'redirectUri' => $redirectUrl,
		]);
	}

	public function getToken(): AccessToken
	{
		return new AccessToken([
			'access_token' => $this->AccessToken,
			'refresh_token' => $this->RefreshToken,
			'expires_in' => $this->Expires,
			'resource_owner_id' => $this->ResourceOwnerId,
		]);
	}


	public function getCMSFields()
	{
		$fields = parent::getCMSFields();
		$fields->removeByName([
			'State',
			'AccessToken',
			'RefreshToken',
			'ResourceOwnerId',
			'Expires',
			'SiteConfigID',
		]);
		$url = $this->getTokenEndpoint();
		$fields->addFieldToTab('Root.Main', LiteralField::create('URL', "<a href='$url'>$url</a>"));
		return $fields;
	}


	public function providePermissions(): array
	{
		$permissions = [
			'MANAGE_INSTAGRAM_TOKENS_ALL' => [
				'name' => 'Manage Instagram tokens',
				'category' => 'Settings'
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null)
	{
		return Permission::check(['MANAGE_INSTAGRAM_TOKENS_ALL']);
	}

	public function canEdit($member = null)
	{
		return Permission::check(['MANAGE_INSTAGRAM_TOKENS_ALL']);
	}

	public function canDelete($member = null)
	{
		return Permission::check(['MANAGE_INSTAGRAM_TOKENS_ALL']);
	}

	public function canCreate($member = null, $context = array())
	{
		return Permission::check(['MANAGE_INSTAGRAM_TOKENS_ALL']);
	}

	private function getTokenEndpoint(): string
	{
		return Controller::join_links([
			Director::absoluteBaseURL(),
			'instagram',
			$this->UserName,
			'access-token',
		]);
	}

}
